#!/usr/bin/env python
from __future__ import generators

import gui
import constants
import widgets
import specializations
import modifiers
import support
import entities

import pygame
from pygame.constants import *

costs = {
    'House' : 250,
    }

class Tile(gui.Paintable):
    """One portion of the map.  You don't place it directly, the
    TileMenu does that for you - you instead specify 'tile locations'"""
    
    def __init__(self, filename, location = (0,0), visible =
                 True):
        gui.Paintable.__init__(self, visible)
        self.tileLoc = location
        self.filename = filename
        self._image = pygame.image.load(filename).convert_alpha()
        self.owner = None # gets set by TileMenu when you put it in
        self.resize( (self._image.get_width(),
                 self._image.get_height()) )
        self.name = "Grass"

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['_image']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self._image = pygame.image.load(dict['filename']).convert_alpha()

    def tileMove(self, newLoc):
        old = self.tileLoc
        self.tileLoc = location
        if self.owner:
            self.owner.recalcAspectFor(self, old)

    def paint(self, screen):
        if self.owner:
            screen.blit(self._image, self.location())
            
class TileMenu(gui.Container):
    """Arranges all the tiles placed into it and lets the user choose
    between them.  If enabled, it traps up/down and left/right."""

    def __init__(self, location, cursorImg, size, visible = True, enabled =
                 True):
        gui.Container.__init__(self, visible, enabled)
        self.move(location)
        self.tiles = None
        self.resize(size)
        self.cursorPos = [0,0]
        self.cursorImg = cursorImg
        self.image = pygame.image.load(cursorImg).convert_alpha()

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['image']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.image = pygame.image.load(dict['cursorImg']).convert_alpha()

    def addPaintable(self, paintable):
        gui.Container.addPaintable(self, paintable)
        paintable.owner = self
        self.recalcAspectFor(paintable)
        
    def event(self, key, pressed, unicode = None):
        if self.focuser and pressed:
            self.direction = key
            if key == pygame.constants.K_RIGHT:
                nextX = self.cursorPos[0] + 1
                if nextX >= self.tileWidth:
                    self.focuser.focusEvent(self)
                else:
                    self.cursorPos[0] = nextX
            elif key == pygame.constants.K_LEFT:
                if self.cursorPos[0] > 0:
                    self.cursorPos[0] -= 1
            elif key == pygame.constants.K_UP:
                if self.cursorPos[1] > 0:
                    self.cursorPos[1] -= 1                
            elif key == pygame.constants.K_DOWN:
                nextY = self.cursorPos[1] + 1
                if nextY >= self.tileHeight:
                    self.focuser.focusEvent(self)
                else:
                    self.cursorPos[1] = nextY
            elif key == K_RETURN:
                self.callback()                

    def recalcAspectFor(self, paintable, old = None):
        """Positions the paintable based on its tile location. If
        specified, 'old' should be the old location of the tile."""
        w, h = paintable.size()
        x, y = paintable.tileLoc

        if self.tiles == None:
            self.tileWidth = self.size()[0] / w
            self.tileHeight = self.size()[1] / h
            self.pixelsPerTileW = w
            self.pixelsPerTileH = h
            self.cursorPos = [0, self.tileHeight-1]


            self.tiles = []
            for xLoc in range(0, self.tileWidth):
                col = [ None for i in range(self.tileHeight) ]
                self.tiles.append(col)

        self.tiles[x][y] = paintable
        if old:
            self.tiles[old[0], old[1]] = None

        x = x * w + self.location()[0]
        y = y * h + self.location()[1]
        paintable.move( (x,y) )
        paintable.resize( (w,h) )
        
    def paint(self, screen):
        for paintable in self._paintables:
            if paintable.isVisible():
                paintable.paint(screen)
        if self.isEnabled():
            x = (self.cursorPos[0] * self.pixelsPerTileW +
                 self.location()[0])
            y = (self.cursorPos[1] * self.pixelsPerTileH +
                 self.location()[1])
            
            screen.blit(self.image, (x,y))

    def replace(self, orig, new = None):
        """Replaces the old paintable with the new one.  If no new one
        is specified,  just removes the old one"""
        self._paintables.remove(orig)
        x = orig.tileLoc[0]
        y = orig.tileLoc[1]
        if self.tiles[x][y] == orig:
            # find something else in paintables that's there so that
            # the tiles array will remain sane
            for tile in self._paintables:
                x2 = tile.tileLoc[0]
                y2 = tile.tileLoc[1]
                if x == x2 and y == y2:
                    self.tiles[x][y] = tile
                    break
            else:
                # Shouldn't ever happen.  Panic
                support.panic("No tiles left!")
        if new:    
            self.add(new)

    def selectedTile(self):
        return self.tileAt( self.cursorPos )

    def tileAt(self, location):
        if self.tiles:
            result = self.tiles[ location[0] ][ location[1] ]
            return result

    def tileGen(self):
        for y in range(0, self.tileHeight):
            for x in range(0, self.tileWidth):
                yield self.tiles[x][y]

    def getBuildings(self):
        for tile in self.tileGen():
            if isinstance(tile, Building):
                yield tile

class TileScreen(gui.TransparentGUIState):
    """Displays a menu of options for a tile"""

    ACTION_NONE = 0
    ACTION_SPECIALIZE = 1
    ACTION_MOVEIN = 2
    ACTION_BUILD = 3
    ACTION_MODIFY = 4
    ACTION_DEMODIFY = 5
    ACTION_LEVELUP = 6
    
    def __init__(self, driver, background, tile):
        gui.TransparentGUIState.__init__(self, driver, background)
        self.tile = tile
        self.font = constants.FONT
        self.substate = None
        self.action = TileScreen.ACTION_NONE
        container = widgets.BorderedContainer([60,60])

        try:
            level = tile.level
            name = "- Lv. %d %s -" % (level, tile.name)
            if tile.resident:
                resident = "(%s's house)" % tile.resident.name
            else:
                resident = ''
        except AttributeError:
            name = "- %s -" % (tile.name)
            resident = ''

        self.title = widgets.Label(self.font, '- %s -' % name)
        container.add(self.title)
        loc = "At: %d, %d" % tile.tileLoc
        container.add(widgets.Label(self.font, loc))
        if resident:
            container.add(widgets.Label(self.font, resident))
        container.add(widgets.Label(self.font, " ")) # space

        menu = self.getMenuFor(tile)

        self.menu = widgets.MenuBox( (100,100), 'arrow.png')
        for item in menu[0]:
            self.menu.add(widgets.Label(self.font, item))

        container.add(self.menu)    

        self.add(container)

        self.menuFunc = menu[1]

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['font']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.font = constants.FONT

    def callback(self, object):
        self.menuFunc(object.getChosenIndex())

    def destroy(self):
        "Destroys whatever's on the current tile"
        self.background.tiles.replace(self.tile)
        self.driver().done()
        
    def getMenuFor(self, tile):
        """Returns a tuple of the form (items for menu, function to
        handle results"""
        try:
            job = tile.getJob()
        except AttributeError:
            job = None
        if job:
            if job == specializations.building:
                # ordinary building
                menu = ["Specialize this building",
                        "Destroy this building",
                        "Nevermind"]
                menuFunc = self.buildingMenu
            elif job == specializations.construction:
                # under construction
                menu = ["Stop construction",
                        "Nevermind"]
                menuFunc = self.constructMenu
            elif job == specializations.mayorHouse:
                # Mayor's House
                menu = ["Upgrade this building",
                        "Nevermind"]
                menuFunc = self.mayorMenu
            else: # Already specialized building
                menu = ["De-specialize this building",
                        "Upgrade this building",
                        "Destroy this building",
                        "Move someone into this building",
                        "Modify this building",
                        "Remove a modification",
                        "Nevermind"]
                menuFunc = self.specialMenu
        else: # just a tile
            menu = ["Build something here",
                    "Nevermind"]
            menuFunc = self.buildMenu

        return (menu, menuFunc)

    def getTown(self):
        return self.background.getTown()

    def reactivate(self):
        if isinstance(self.substate, support.Confirm):
            if self.substate.result == 0:
                self.destroy()
        elif self.action == TileScreen.ACTION_SPECIALIZE:
            if self.substate.result != None:
                self.specializeDo(self.substate.result)
        elif self.action == TileScreen.ACTION_MOVEIN:
            if self.substate.result != None:
                self.tile.moveIn(self.substate.result)
                self.driver().done()
        elif self.action == TileScreen.ACTION_BUILD:
            result = self.substate.result
            if result == 0:
                buildX = self.tile.tileLoc[0]
                buildY = self.tile.tileLoc[1]
                newBuild = Construction( (buildX, buildY) )
                self.background.tiles.add(newBuild)
            self.driver().done()
        elif self.action == TileScreen.ACTION_MODIFY:
            chosen = self.substate.result
            if chosen != None:
                newMod = modifiers.buildingMods[chosen][0]
                self.tile.modifiers.append(newMod)
            self.driver().done()
        elif self.action == self.ACTION_DEMODIFY:
            if self.substate.result != None:
                chosen = self.options[self.substate.result]
                self.tile.modifiers.remove(chosen)
        elif self.action == self.ACTION_LEVELUP:
            if self.substate.result != None:
                self.tile.level += 1
                self.driver().done()

            
    def specialize(self):
        options = []
        self.action = TileScreen.ACTION_SPECIALIZE
        for option, price in specializations.buildingSpecs:
            options.append( (option.name, price) )
        options.append( ('Nevermind', 0))
        caption = 'What do you want to make this building?'
        self.substate = support.ShopState(self.driver(), self,
                                          caption, options)
        self.driver().start(self.substate)

    def specializeDo(self, chosen):
        try:
            newSpec = specializations.buildingSpecs[chosen][0]
        except IndexError:
            # canceled out
            return
        self.tile.setJob(newSpec)
        self.tile.name = newSpec.name
        self.driver().done()

    def upgrade(self):
        options = [
            ( 'Level %d' % (self.tile.level + 1),
              (self.tile.level + 1) * constants.LEVELMULT ),
            ( 'Nevermind', 0) ]
        self.action = self.ACTION_LEVELUP
        caption = "Upgrade to what level?"
        self.substate = support.ShopState(self.driver(),self,
                                          caption, options)
        self.driver().start(self.substate)

    # VARIOUS MENUS FOLLOW

    def buildMenu(self, chosen):
        if chosen == 0:
            options = []
            self.action = self.ACTION_BUILD
            options.append( ('House', costs['House']) )

            options.append( ('Nevermind', 0) )
            caption = 'What do you want to build here?'
            self.substate = support.ShopState(self.driver(), self,
                                              caption, options)
            self.driver().start(self.substate)
        else:
            self.driver().done()

    def buildingMenu(self, chosen):
        if chosen == 0:
            self.specialize()
        elif chosen == 1:
            self.substate = support.Confirm(self.driver(), self,
                                            "Really tear it down?")
            self.driver().start(self.substate)
        else:
            self.driver().done()

    def mayorMenu(self, chosen):
        if chosen == 0:
            self.upgrade()
            pass
        else:
            self.driver().done()

    def constructMenu(self, chosen):
        if chosen == 0:
            self.substate = support.Confirm(self.driver(), self,
                                            "Really stop construction?")
            self.driver().start(self.substate)
        else:
            self.driver().done()

    def specialMenu(self, chosen):
        if chosen == 0:
            newSpec = specializations.building
            self.tile.setJob(newSpec)
            self.tile.name = newSpec.name
            self.driver().done()
        elif chosen == 1:
            self.upgrade()
        elif chosen == 2:
            self.substate = support.Confirm(self.driver(), self,
                                            "Really tear it down?")
            self.driver().start(self.substate)
        elif chosen == 3:
            caption = 'Who do you want to move in?'
            self.action = self.ACTION_MOVEIN
            self.substate = entities.TownspeopleChooser(self.driver(),
                                                        self, caption)
            self.driver().start(self.substate)
        elif chosen == 4:
            options = []
            self.action = TileScreen.ACTION_MODIFY
            for option, price in modifiers.buildingMods:
                if not(option in self.tile.modifiers):
                    options.append( (option.name, price) )
            if options:
                caption = 'What modification do you wish to add?'
            else:
                caption = ('There are no additional modifications you' +
                           ' can make')
            options.append( ('Nevermind', 0))
            self.substate = support.ShopState(self.driver(), self,
                                              caption, options)
            self.driver().start(self.substate)
        elif chosen == 5:
            self.action = TileScreen.ACTION_DEMODIFY
            self.options = []
            options = []
            for mod in self.tile.modifiers:
                self.options.append(mod)
                options.append( (mod.name, 500))
            if options:
                caption = 'Which modifications do you wish to remove?'
            else:
                caption = 'There are no modifications to remove!'
            options.append( ('Nevermind', 0))
            self.substate = support.ShopState(self.driver(), self,
                                              caption, options)
            self.driver().start(self.substate)
        
        else:
            self.driver().done()
        
class Building(Tile):
    def __init__(self, filenames, name, job, location = (0,0), visible
                 = True):
        "Filenames is a list of files to draw, in the given order"
        Tile.__init__(self, filenames[0], location, visible)

        self.level = 1
        self.name = name
        self.setJob(job)
        self.resident = None
        self.modifiers = []

        self.images = []
        self.filenames = filenames
        for filename in filenames:
            self.images.append(pygame.image.load(filename).convert_alpha())

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['images']
        del odict['_image']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.images = []
        for filename in dict['filenames']:
            self.images.append(pygame.image.load(filename).convert_alpha())
        self._image = self.images[0]

    def canHold(self):
        return self.job.canHold()

    def getJob(self):
        return self.job

    def getName(self):
        return self.name

    def levelUp(self):
        self.level += 1

    def moveIn(self, person):
        if self.resident != None:
            self.resident.residence = None
        self.resident = person
        
        if person.residence != None:
            person.residence.resident = None
        person.residence = self

    def paint(self, screen):
        if self.owner:
            for image in self.images:
                screen.blit(image, self.location())

    def setJob(self, newJob):
        self.job = newJob

## VARIOUS BUILDING TYPES FOLLOW ##

class MayorHouse(Building):
    def __init__(self, location = (0,0), visible = True):
        Building.__init__(self, ('house.png',), "Mayor's House",
                          specializations.mayorHouse, location,
                          visible)

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['_image']
        del odict['images']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self._image = pygame.image.load(dict['filename']).convert_alpha()
        self.images = []
        for filename in dict['filenames']:
            self.images.append(pygame.image.load(filename).convert_alpha())

class Construction(Building):
    def __init__(self, location = (0,0), visible = True):
        Building.__init__(self, ('house.png',), "Construction",
                          specializations.construction, location,
                          visible)
        
        self.font = constants.FONT
        self.setImg()

    def setImg(self):
        self.curImg = self.font.render('%d' % (8-self.level), 0,
                                       (255, 255, 255))

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['_image']
        del odict['images']
        del odict['font']
        del odict['curImg']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self._image = pygame.image.load(dict['filename']).convert_alpha()
        self.images = []
        for filename in dict['filenames']:
            self.images.append(pygame.image.load(filename).convert_alpha())
        self.font = constants.FONT
        self.setImg()


    def levelUp(self):
        Building.levelUp(self)
        self.setImg()
        
    def paint(self, screen):
        Building.paint(self, screen)
        screen.blit(self.curImg, self._aspect)
