#!/usr/bin/env python

import random
import pygame

import gui
import constants
import specializations
import widgets
import support
import modifiers

LASTNAMES = [ 'Abbey', 'Abbot', 'Abdul', 'Abraham', 'Ackerman',
              'Adams', 'Addison', 'Ader', 'Agnew', 'Akers', 'Alban',
              'Alford', 'Allen', 'Allgeier', 'Alston', 'Alvarez',
              'Ambrose', 'Anchor', 'Anderson', 'Andrews', 'Applebaum',
              'Armstrong', 'Ashcroft', 'Atkinson', 'Bailey', 'Baird',
              'Baker', 'Balog', 'Barker', 'Barnes', 'Barron', 'Bates',
              'Bauer', 'Beale', 'Beaumont', 'Beeker', 'Bell',
              'Bender', 'Bennett', 'Bierly', 'Bird', 'Blackwell',
              'Blake', 'Bogar', 'Bond', 'Booker', 'Bowers', 'Boyd',
              'Brandt', 'Braun', 'Briscoe', 'Brooks', 'Brown',
              'Bryce', 'Burke', 'Burns', 'Butler', 'Cain', 'Callahan',
              'Campbell', 'Caplan', 'Carlson', 'Carpenter', 'Carroll',
              'Carver', 'Castel', 'Chadbourne', 'Chambers', 'Chapman',
              'Clarke', 'Clemens', 'Coburn', 'Cohen', 'Cole',
              'Collins', 'Connor', 'Conway', 'Cooper', 'Corrigan',
              'Cowell', 'Crawford', 'Dales', 'Daniels', 'Davenport',
              'Davis', 'Day', 'Dean', 'Decker', 'Dempsey'
              ]

FIRSTNAMES = [ 'Adam', 'Alex', 'Chad', 'Roger', 'Brian',
               'Michelle', 'Barbara', 'Jessica', 'Joan']

VILLAINS = [ 'Morbo', 'Dr. Why', 'Prof. Chaos', 'XYZZY',
             'Globzor', 'Krefk', 'Ultimor', 'Bob']

class Entity(gui.Paintable):

    def __init__(self, filename):
        gui.Paintable.__init__(self, True)
        self._image = pygame.image.load(filename).convert_alpha()
        self.name = 'Generic Entity'
        self.filename = filename
        self.level = 1
        self.picky = 1
        self.align = 0.5
        import constants
        self.font = constants.FONT
        self.calcSize()
        self.modifiers = []

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['_image']
        del odict['font']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self._image = pygame.image.load(self.filename).convert_alpha()
        self.font = constants.FONT

    def calcSize(self):
        font = self.font
        w = self._image.get_width() + 2 + font.size(constants.NAME)[0]
        h = max(self._image.get_height(), font.size(constants.NAME)[1] * 2)
        self.resize( (w,h) )
        
    def getName(self):
        return self.name

    def getLevel(self):
        return self.level

    def paint(self, screen):
        font = self.font
        # draw character image
        screen.blit(self._image, self.location())

        # draw name and level
        x, y = self.location()
        newX = x + self._image.get_width() + 2
        newY = y + font.size(self.name)[1] + 2
        text = font.render(self.name, 0, (255, 255, 255))
        screen.blit(text, (newX, y))
        level = 'Lv %d %s' % (self.level, self.job.name)
        text = font.render(level, 0, (255, 255, 255))
        screen.blit(text, (newX, newY))

    def setJob(self, newJob):
        self.job = newJob

class Townsperson(Entity):
    def __init__(self, filename):
        Entity.__init__(self, filename)
        self.name = random.choice(LASTNAMES)
        self.calcSize()
        self.setJob(specializations.townsperson)
        self.residence = None

    def __str__(self):
        return self.name + ' Level ' + str(self.level)

    def __repr__(self):
        return self.__str__()

class Mayor(Townsperson):
    def __init__(self, name):
        Townsperson.__init__(self, 'townsman.png')
        self.name = name
        self.setJob(specializations.mayor)

class Adventurer(Entity):
    def __init__(self, filename):
        Entity.__init__(self, filename)
        self.name = random.choice(FIRSTNAMES)
##        self.setJob(specializations.adventurer)
        self.wanderlust = random.random()
        self.distance = random.randrange(100)
        self.away = 0
        self.here = 0
        self.calcSize()

    def __str__(self):
        return self.name + ' Level ' + str(self.level)

    def __repr__(self):
        return self.__str__()


class Villain(Entity):
    def __init__(self):
        Entity.__init__(self, 'villain.png')
        self.name = random.choice(VILLAINS)
        self.calcSize()
        self.setJob(random.choice(specializations.villainSpecs))
        self.away = 0
        self.here = 0
                    
    def __str__(self):
        return self.name + ' Level ' + str(self.level)

    def __repr__(self):
        return str(self)

class PersonScreen(gui.TransparentGUIState):
    """Menu of options for a person"""

    ACTION_NONE = 0
    ACTION_TRAIN = 1
    ACTION_MODIFY = 2
    ACTION_DEMODIFY = 3
    ACTION_LEVELUP = 4
    ACTION_KICK = 5

    def __init__(self, driver, background, person):
        gui.TransparentGUIState.__init__(self, driver, background)
        self.person = person
        self.font = constants.FONT
        self.substate = None
        self.action = PersonScreen.ACTION_NONE

        container = widgets.BorderedContainer( [60,60] )

        name = '-- %s, a level %d %s --' % (self.person.name,
                                            self.person.level,
                                            self.person.job.name)
        self.title = widgets.Label(self.font, name)
        container.add(self.title)
        container.add(widgets.Label(self.font, " ")) # space

        self.menu = widgets.MenuBox( (100,100), 'arrow.png')
        
        if self.person.job == specializations.townsperson:
            text = 'Train this person'
        else:
            text = 'Re-train this person'
        self.menu.add(widgets.Label(self.font, text))
        text = 'Level this person up'
        self.menu.add(widgets.Label(self.font, text))

        text = 'Modify this person'
        self.menu.add(widgets.Label(self.font, text))
        text = 'Remove a modification'
        self.menu.add(widgets.Label(self.font, text))

        text = 'Kick this person out of town'
        self.menu.add(widgets.Label(self.font, text))

        self.menu.add(widgets.Label(self.font, 'Nevermind'))        
        container.add(self.menu)
        self.add(container)
    
    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['font']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.font = constants.FONT



    def callback(self, object):
        chosen = object.getChosenIndex()
        if chosen == 0:
            options = []
            if self.person.job == specializations.mayor:
                caption = ("You're Mayor-For-Life, in case you've "+
                           "forgotten.")
                text = 'Oh yeah.'
                options.append( (text, 0))
                self.action = PersonScreen.ACTION_NONE
            else:
                for option, price in specializations.personSpecs:
                    options.append( (option.name, price) )
                options.append( ('Nevermind', 0))
                self.action = PersonScreen.ACTION_TRAIN
                caption = "Train this person to be what?"
            self.substate = support.ShopState(self.driver(), self,
                                              caption, options)
            self.driver().start(self.substate)

        elif chosen == 2:
            options = []
            self.action = self.ACTION_MODIFY
            for option, price in modifiers.personMods:
                if not(option in self.person.modifiers):
                    options.append( (option.name, price) )
            if options:
                caption = 'What modification do you wish to add?'
            else:
                caption = ('There are no additional modifications you' +
                           ' can make')
            options.append( ('Nevermind', 0))
            self.substate = support.ShopState(self.driver(), self,
                                              caption, options)
            self.driver().start(self.substate)
        elif chosen == 3:
            self.action = self.ACTION_DEMODIFY
            self.options = []
            options = []
            for mod in self.person.modifiers:
                self.options.append(mod)
                options.append( (mod.name, 500))
            if options:
                caption = 'Which modifications do you wish to remove?'
            else:
                caption = 'There are no modifications to remove!'
            options.append( ('Nevermind', 0))
            self.substate = support.ShopState(self.driver(), self,
                                              caption, options)
            self.driver().start(self.substate)
            
        elif chosen == 1:
            options = [
                ( 'Level %d' % (self.person.level + 1),
                  (self.person.level + 1) * constants.LEVELMULT ),
                ( 'Nevermind', 0) ]
            self.action = self.ACTION_LEVELUP
            caption = "Upgrade to what level?"
            self.substate = support.ShopState(self.driver(),self,
                                              caption, options)
            self.driver().start(self.substate)

        elif chosen == 4:
            self.action = self.ACTION_KICK
            if self.person.job != specializations.mayor:
                self.substate = support.Confirm(self.driver(), self,
                                                "Give 'em the boot?")
                self.driver().start(self.substate)
        else:
            self.driver().done()

    def getTown(self):
        return self.background.getTown()

    def reactivate(self):
        if self.action == self.ACTION_TRAIN:
            if self.substate.result:
                self.train(self.substate.result)
        elif self.action == self.ACTION_MODIFY:
            chosen = self.substate.result
            if chosen != None:
                newMod = modifiers.personMods[chosen][0]
                self.person.modifiers.append(newMod)
                self.driver().done()
        elif self.action == self.ACTION_DEMODIFY:
            if self.substate.result != None:
                chosen = self.options[self.substate.result]
                self.person.modifiers.remove(chosen)
        elif self.action == self.ACTION_LEVELUP:
            if self.substate.result != None:
                self.person.level += 1
                self.driver().done()
        elif self.action == self.ACTION_KICK:
            if self.substate.result == 0:
                self.background.removeTownsperson(self.person)
                self.driver().done()
            
    def train(self, chosen):
        try:
            newSpec = specializations.personSpecs[chosen][0]
        except IndexError:
            # canceled out
            return
        self.person.setJob(newSpec)
        self.driver().done()

class TownspeopleChooser(gui.TransparentGUIState):
    """Allows the player to choose a townsperson (except for the
    mayor)"""

    def __init__(self, driver, background, caption):
        """Like ShopState, this expects background to have a getTown
        function"""
        gui.TransparentGUIState.__init__(self, driver, background)

        self.font = constants.FONT
        self.result = None
        self.caption = caption
        focus = []

        container = widgets.BorderedContainer( [100,60] )

        container.add(widgets.Label(self.font, caption))
        container.add(widgets.Label(self.font, " ")) # space

        self.exitMenu = widgets.MenuBox( (100,100), 'arrow.png')
        self.exitMenu.align = widgets.MenuBox.HORIZONTAL
        self.exitMenu.add(widgets.Label(self.font, 'Nevermind'))
        focus.append(self.exitMenu)
        container.add(self.exitMenu)

        self.potentials = filter(specializations.isNotMayor,
                                 background.getTown().townspeople)
        self.incomingMenu = widgets.MenuBox( (100,100), 'arrow.png')
        self.incomingMenu.align = widgets.MenuBox.HORIZONTAL
        self.incomingMenu.xSpacing = 20
        self.incomingMenu.xStretch = 0
        self.incomingMenu.allowScrolling = 1
        self.incomingMenu.resize( ( (container.size()[0] - 10),
                                    40))
        for person in self.potentials:
            self.incomingMenu.add(person)

        if len(self.potentials) > 0:
            container.add(self.incomingMenu)
            focus.append(self.incomingMenu)
        else:
            text = "Nobody qualifies to move in!"
            container.add(widgets.Label(self.font, text))

        self.container = container
        self.add(container)
        self.focus = gui.VerticalFocuser( focus )

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['font']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.font = constants.FONT


    def callback(self, object):
        if object == self.incomingMenu:
            self.result = self.incomingMenu.getChosen()
        # Major problem - using the people moved their pictures
        # around.  Reset the townspeople menu to get them back.
        town = self.background.getTown()
        town.townMenu.move(town.townMenu.location())
        self.driver().done()

def randomTownsperson():
    files = ['townsman.png', 'townswoman.png']
    result = Townsperson(random.choice(files))
    return result
