#!/usr/bin/env python
"""The GUI states as well as Paintable and Eventable are defined here"""

from states import State

import pygame
import pygame.constants

class Eventable:
    MASK_ACCEPT = -1
    MASK_NONE   =  0
    MASK_DENY   =  1
    """An Eventable is an object which responds to keypresses"""
    def __init__(self, enabled = True):
        self._enabled = enabled
        self._mask = Eventable.MASK_NONE
        self._chars = []
        self.parent = None
        self.squelched = False

    def callback(self):
        """Shorthand for self.parent.callback(self) but checks to make
        sure there is a parent first"""
        if self.parent:
            self.parent.callback(self)

    def disable(self):
        self._enabled = False

    def enable(self):
        self._enabled = True

    def event(self, key, pressed, unicode = None):
        """Subclasses should override this to provide behavior based
        on the key."""
        pass

    def isEnabled(self):
        return self._enabled

    def isSquelched(self):
        """Returns whether we are squelched or not.  If we were,
        return so and then disable the squelch."""
        if self.squelched:
            self.squelched = False
            return True
        return False

    def setMask(self, type, mask):
        """Sets this eventable to only respond to characters in (or
        not in, depending on the value of type) mask"""
        self._mask = type
        self._chars = mask

    def setParent(self, parent):
        self.parent = parent

    def squelch(self):
        """Disables this component for one cycle - used to suppress
        the cascading of key events when a focuser switches focus"""
        self.squelched = True

class Paintable:
    """A Paintable is a basic object which can be painted"""
    def __init__(self, visible = True):
        self._aspect = [0, 0, 0, 0]
        self._visible = visible

    def getAspect(self):
        return self._aspect

    def hide(self):
        self._visible = False

    def isVisible(self):
        return self._visible

    def location(self):
        return (self._aspect[0], self._aspect[1])

    def move(self, newLoc):
        self._aspect[0] = newLoc[0]
        self._aspect[1] = newLoc[1]        

    def paint(self, screen):
        """Subclasses should override this to paint how they want
        to"""
        pass

    def resize(self, newSize):
        self._aspect[2] = newSize[0]
        self._aspect[3] = newSize[1]

    def size(self):
        return (self._aspect[2], self._aspect[3])

    def show(self):
        self._visible = True

class GUIState(State):
    """A state where user interaction can take place"""
    def __init__(self, driver):
        State.__init__(self, driver)
        self._paintables = []
        self._eventables = []

    def add(self, widget):
        """This function is a wrapper around addEventable and
        addPaintable.  It will add widget to the appropriate lists
        depending on what class(es) it is derived from"""
        if isinstance(widget, Eventable):
            self.addEventable(widget)
        if isinstance(widget, Paintable):
            self.addPaintable(widget)

    def addEventable(self, eventable):
        self._eventables.append(eventable)
        eventable.setParent(self)

    def addPaintable(self, paintable):
        self._paintables.append(paintable)

    def callback(self, object):
        """When something happens to eventables, they'll call this
        function.  It's the subclass' responsibility to do something
        about it"""
        pass

    def event(self, key, pressed, unicode = None):
        for eventable in self._eventables:
            if eventable.isEnabled() and not eventable.isSquelched():
                eventable.event(key, pressed, unicode)

    def paint(self, screen):
        for paintable in self._paintables:
            if paintable.isVisible():
                paintable.paint(screen)

    def removeEventable(self, eventable):
        self._eventables.remove(eventable)

    def removePaintable(self, paintable):
        self._paintables.remove(paintable)

class Container(Paintable, Eventable):
    """This is an object that can contain other objects.  Note that it
    is intended to  be an abstract class; use a subclass if you
    actually want to see or interact with what you add to it.
    Subclasses should manage the placement of all paintables
    themselves."""
    def __init__(self, visible = True, enabled = True):
        Paintable.__init__(self, visible)
        Eventable.__init__(self, visible)
        self._eventables = []
        self._paintables = []

    def __len__(self):
        return len(self._paintables)

    def add(self, widget):
        """This function is a wrapper around addEventable and
        addPaintable.  It will add widget to the appropriate lists
        depending on what class(es) it is derived from"""
        if isinstance(widget, Eventable):
            self.addEventable(widget)
        if isinstance(widget, Paintable):
            self.addPaintable(widget)
                    
    def addEventable(self, eventable):
        eventable.setParent(self)
        self._eventables.append(eventable)

    def addPaintable(self, paintable):
        self._paintables.append(paintable)

    def callback(self, object = None):
        if self.parent:
            if object:
                self.parent.callback(object)
            else:
                self.parent.callback(self)

    def clear(self):
        self._paintables = []

    def event(self, key, pressed, unicode = None):
        for eventable in self._eventables:
            if eventable.isEnabled() and not eventable.isSquelched():
                eventable.event(key, pressed, unicode)

    def move(self, newLoc):
        Paintable.move(self, newLoc)
        self.refresh()

    def refresh(self):
        items = self._paintables
        self.clear()
        for item in items:
            self.add(item)

    def paint(self, screen):
        """Location of children can be done one of two ways: when
        they're added, or when they're painted.  If you're doing the
        second, override this function."""
        for paintable in self._paintables:
            paintable.paint(screen)

    def remove(self, widget):
        if isinstance(widget, Eventable):
            self.removeEventable(widget)
        if isinstance(widget, Paintable):
            self.removePaintable(widget)
            self.refresh()

    def removeEventable(self, eventable):
        self._eventables.remove(eventable)

    def removePaintable(self, paintable):
        self._paintables.remove(paintable)

class TransparentGUIState(GUIState):
    """A GUIState which paints another state as a background before
    painting itself."""
    def __init__(self, driver, background):
        GUIState.__init__(self, driver)
        self.background = background

    def paint(self, screen):
        self.background.paint(screen)
        GUIState.paint(self, screen)

class Focuser:
    """This allows only one of its children to be enabled at
    once. Designed to be overridden for specific focusing needs"""
    def __init__(self):
        self.eventables = []

    def add(self, eventable):
        eventable.disable()
        eventable.focuser = self
        self.eventables.append(eventable)

    def focus(self, eventable):
        for x in self.eventables:
            x.disable()
        eventable.enable()
        eventable.squelch()

    def focusEvent(self, initiator):
        """This is to be overridden - eventables will call this
        function to indicate that they wish to give up their
        focus to someone else"""
        pass

    def remove(self, object):
        self.eventables.remove(object)
        if object.isEnabled():
            self.focus(self.eventables[0])

class VerticalFocuser(Focuser):
    def __init__(self, items):
        Focuser.__init__(self)
        for item in items:
            self.add(item)
        self.eventables[0].enable()
        self.enabled = 0

    def focusEvent(self, initiator):
        direction = initiator.direction
        if direction == pygame.constants.K_DOWN:
            self.enabled += 1
        elif direction == pygame.constants.K_UP:
            self.enabled -= 1
        if self.enabled < 0:
            self.enabled += len(self.eventables)
        elif self.enabled >= len(self.eventables):
            self.enabled -= len(self.eventables)
        try:
            self.focus(self.eventables[self.enabled])
        except IndexError:
            self.focus(self.eventables[0])
            self.enabled = 0
