#!/usr/bin/env python

import constants
import modifiers

class Specialization:
    def __init__(self, name):
        "Initialize"
        self.name = name
        
    def effectiveAlignment(self, employer):
        """If the object with this specialization has no inherent
        alignment (i.e. a building) then this represents the alignment
        that the given specialization gives it.  If it does have an
        alignment, this returns the effective alignment this
        specialization and modifiers give it"""
        try:
            finalMod = employer.align
        except AttributeError:
            return None
        
        for mod in employer.modifiers:
            finalMod *= mod.alignMod()
        return finalMod

    def effectiveLevel(self, employer):
        """Returns the effective level of this specification, taking
        into account modifiers"""
        finalMod = 1.0
        for mod in employer.modifiers:
            finalMod *= mod.levelMod()
        return finalMod * employer.level

    def comingToEffect(self, prospect, employer):
        """Return a number representing the effect
        that this specialization has on the prospect's chance of
        coming here"""
        return constants.NONE

    def canHold(self):
        """Answer the number of townspeople a building with this job
        can hold."""
        return 0

    def getModsFor(self, modString, employer):
        finalMod = 1.0
        for mod in employer.modifiers:
            modFunc = getattr(mod, modString)
            finalMod *= modFunc()
##            finalMod *= mod.modFunc()
        return finalMod

    def upkeep(self, employer):
        """Money per-turn that it takes to keep this running"""
        return (employer.level * constants.EXPENSEMULT *
                self.getModsFor('upkeepMod', employer))

    def income(self, prospect, employer):
        """Money per-turn that this job will earn"""
        return 0

class Building(Specialization):
    def __init__(self):
        Specialization.__init__(self, "House")

    def canHold(self):
        return 3

class Construction(Specialization):
    def __init__(self):
        Specialization.__init__(self, "House (Under Construction)")

    def upkeep(self, employer):
        return 0

class MayorHouse(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Mayor's House")

    def canHold(self):
        return 1

    def comingToEffect(self, prospect, employer):
        return constants.HI

    def income(self, prospect, employer):
        mods = self.getModsFor('incomeMod', employer)
        return prospect.level / 2 * constants.INCOMEMULT * mods

class Inn(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Inn")

    def canHold(self):
        return 1

    def comingToEffect(self, prospect, employer):
        resident = employer.resident
        if resident:
            if resident.job == innkeeper:
                return constants.LOW
        return constants.NONE

    def income(self, prospect, employer):
        resident = employer.resident
        mods = self.getModsFor('incomeMod', employer)
        if resident:
            if resident.job == innkeeper:
                return prospect.level * constants.INCOMEMULT * mods
        return 0

class TrainingHall(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Training Hall")

    def canHold(self):
        return 1

    def comingToEffect(self, prospect, employer):
        resident = employer.resident
        if resident:
            if resident.job == trainer:
                return constants.AVG
            else:
                return constants.LOW
        else:
            return constants.NONE

    def income(self, prospect, employer):
        resident = employer.resident
        mods = self.getModsFor('incomeMod', employer)
        if resident:
            if resident.job == trainer:
                return prospect.level * INCOMEMULT * mods
            else:
                return prospect.level * INCOMEMULT / 2 *  mods
        else:
            return 0

class Shop(Specialization):
    ITEM = 1
    ARMOR = 2
    WEAPON = 3
    MAGIC = 4
    TEMPLE = 5
    BLACKMARKET = 6

    NAMEDICT = {
        ITEM : 'Item Shop',
        ARMOR : 'Armor Shop',
        WEAPON : 'Weapon Shop',
        MAGIC : 'Magic Shop',
        TEMPLE : 'Temple',
        BLACKMARKET : 'Black Market',
    }
    
    def __init__(self, type):
        Specialization.__init__(self, Shop.NAMEDICT[type] )
        self.type = type

    def effectiveAlignment(self, employer):
        if self.type == self.TEMPLE:
            return 1
        elif self.type == self.BLACKMARKET:
            return 0
        else:
            return Specialization.effectiveAlignment(self, employer)

    def canHold(self):
        return 1

    def comingToEffect(self, prospect, employer):
        base = constants.LOW
        if (prospect.job == priest and
            self.type==self.TEMPLE):
            base =constants.HI
        elif (prospect.job == thief and
              self.type==self.BLACKMARKET):
            base = constants.HI
        elif (prospect.job == warrior and
              (self.type == self.ARMOR or
               self.type == self.WEAPON)):
            base = constants.HI
        elif (prospect.job == wizard and
              (self.type == self.MAGIC)):
            base = constants.HI
        elif (prospect.job == wizard):
            base = constants.LOW / 2
        
        resident = employer.resident
        if resident:
            if resident.job == shopkeeper:
                return base
        return constants.NONE

    def upkeep(self, employer):
        mods = self.getModsFor('upkeepMod', employer)
        if self.type == self.BLACKMARKET:
            return 10 * employer.level* constants.EXPENSEMULT * mods
        elif self.type == self.MAGIC:
            return 3 * employer.level* constants.EXPENSEMULT * mods
        else:
            return employer.level* constants.EXPENSEMULT * mods

    def income(self, prospect, employer):
        resident = employer.resident
        mods = self.getModsFor('incomeMod', employer)
        if resident:
            if resident.job == shopkeeper:
                if self.type == self.MAGIC:
                    return 3 * prospect.level * constants.INCOMEMULT * mods
                elif self.type == self.TEMPLE:
                    return 2 * prospect.level * constants.INCOMEMULT * mods
                elif self.type == self.BLACKMARKET:
                    return 5 * prospect.level * constants.INCOMEMULT * mods
                else:
                    return prospect.level * constants.INCOMEMULT * mods
        return 0
                    
class Arena(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Arena")

    def canHold(self):
        return 1

    def comingToEffect(self, prospect, employer):
        return constants.AVG

    def upkeep(self, employer):
        mods = self.getModsFor('upkeepMod', employer)
        return 5 * employer.level* constants.EXPENSEMULT * mods

    def income(self, prospect, employer):
        mods = self.getModsFor('incomeMod', employer)
        return 2 * prospect.level * constants.INCOMEMULT * mods

# People classes
class Townsperson(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Townsperson")

    def upkeep(self, employer):
        return 0


class Mayor(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Mayor")

    def comingToEffect(self, prospect, employer):
        return constants.HI

    def upkeep(self, employer):
        return 0


class Beggar(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Beggar")

    def comingToEffect(self, prospect, employer):
        return -(constants.LOW)

    def upkeep(self, employer):
        return 0

    def income(self, prospect, employer):
        mods = self.getModsFor('incomeMod', employer)
        alignDiff = abs(employer.align - prospect.align)
        if alignDiff < prospect.picky:
            result = (prospect.picky - alignDiff /
                      float(prospect.picky)) * prospect.level
        return int(result * mods)

class Innkeeper(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Innkeeper")

    def upkeep(self, employer):
        return 0


class Shopkeeper(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Shopkeeper")

    def upkeep(self, employer):
        return 0


class Informer(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Informer")

    def comingToEffect(self, prospect, employer):
        return constants.LOW

    def upkeep(self, employer):
        return 0


class Presenter(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Presenter")

    def comingToEffect(self, prospect, employer):
        return constants.HI

    def upkeep(self, employer):
        mods = self.getModsFor('upkeepMod', employer)
        return employer.level * 3* constants.EXPENSEMULT * mods

class Trainer(Specialization):
    def __init__(self):
        Specialization.__init__(self, "Trainer")

    def upkeep(self, employer):
        return 0

# Adventurer classes

class Adventurer(Specialization):
    def __init__(self, name):
        Specialization.__init__(self, name)

    def upkeep(self, employer):
        return 0

class Warrior(Adventurer):
    def __init__(self):
        Adventurer.__init__(self, "Warrior")

class Wizard(Adventurer):
    def __init__(self):
        Adventurer.__init__(self, "Wizard")

class Priest(Adventurer):
    def __init__(self):
        Adventurer.__init__(self, "Priest")

class Thief(Adventurer):
    def __init__(self):
        Adventurer.__init__(self, "Thief")

# Villain classes
class Destroyer(Specialization):
    def __init__(self):
        Specialization.__init__(self, 'Destroyer')

class Evil(Specialization):
    def __init__(self):
        Specialization.__init__(self, 'Evilbringer')

class Benign(Specialization):
    def __init__(self):
        Specialization.__init__(self, 'Benign')

class Conqueror(Specialization):
    def __init__(self):
        Specialization.__init__(self, 'Conqueror')


# INSTANCES FOLLOW

# Building specs
building = Building()
construction = Construction()
mayorHouse = MayorHouse()
itemShop = Shop(Shop.ITEM)
armorShop = Shop(Shop.ARMOR)
weaponShop = Shop(Shop.WEAPON)
magicShop = Shop(Shop.MAGIC)
temple = Shop(Shop.TEMPLE)
blackMarket = Shop(Shop.BLACKMARKET)
inn = Inn()
arena = Arena()
trainingHall = TrainingHall()

buildingSpecs = [
    (itemShop,    100),
    (armorShop,   200),
    (weaponShop,  200),
    (magicShop,   500),
    (temple,      400),
    (blackMarket, 500),
    (inn,         100),
    (arena,       750),
    (trainingHall,300) ]

# Person specs
townsperson = Townsperson()
mayor = Mayor()
adventurer = Adventurer('Adventurer')
beggar = Beggar()
innkeeper = Innkeeper()
shopkeeper = Shopkeeper()
informer = Informer()
presenter = Presenter()
trainer = Trainer()

personSpecs = [
    (townsperson, 0),
    (beggar,     50),
    (innkeeper, 100),
    (shopkeeper,100),
    (informer,  150),
    (presenter, 100),
    (trainer,   150),
    ]

# Adventurer specs
warrior = Warrior()
wizard = Wizard()
priest = Priest()
thief = Thief()

adventSpecs = [
    (warrior, 'adventurer.png'),
    (wizard, 'adventurer.png'),
    (priest, 'adventurer.png'),
    (thief, 'adventurer.png') ]

# Villain specs
destroyer = Destroyer()
evil = Evil()
benign = Benign()
conqueror = Conqueror()

villainSpecs = [
    destroyer,
    evil,
    benign,
    conqueror ]

# Functions
def isNotMayor(entity):
    return entity.job != mayor
