#!/usr/bin/env python
"""This provides basic state management"""

import pygame
from pygame.locals import *

class StateDriver:
    "The StateDriver runs the application"
    def __init__(self, screen):
        self._states = []
        self._screen = screen

    def done(self):
        "The current state calls this when it has finished"
        self._states.pop()
        self.getCurrentState().reactivate()

    def getCurrentState(self):
        """Call if you need the current state. If there is none, quit."""
        try:
            return self._states[len(self._states) - 1]
        except IndexError:
            raise SystemExit  # we're done if theren't any states left

    def getScreenSize(self):
        return self._screen.get_size()

    def quit(self):
        raise SystemExit

    def replace(self, state):
        "Shortcut for pop() then start()"
        self._states.pop()
        self.start(state)

    def run(self):
        "Call this when you've put the initial state onto the stack"
        currentState = self.getCurrentState()
        while(currentState):
            # poll queue
            event = pygame.event.poll()
            while(event.type != NOEVENT):
                if event.type == QUIT:
                    currentState = None
                    break
                elif event.type == KEYUP or event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        #currentState = None
                        #break
                        pass
                    if event.type == KEYUP:
                        currentState.event(event.key, 0)
                    if event.type == KEYDOWN:
                        currentState.event(event.key, 1, event.unicode)
                event = pygame.event.poll()
#            self._screen.fill( (0, 0, 0) )
            if currentState:
                #print currentState
                currentState.paint(self._screen)
                currentState.update()
                
                currentState = self.getCurrentState()
                
                pygame.display.flip()
                pygame.time.delay(40);

    def start(self, state):
        """Makes the given state the currently-running state, pushing
        all other states down the stack."""
        self._states.append(state)
        self.getCurrentState().activate()

class State:
    def __init__(self, driver):
        self._driver = driver

    def activate(self):
        """Called by the statedriver when another state starts us"""
        pass

    def driver(self):
        return self._driver

    def event(self, key, pressed, unicode = None):
        """Called by the statedriver when a key is pressed"""
        pass

    def paint(self, screen):
        """Called by the statedriver when we need to paint ourselves"""
        pass

    def reactivate(self):
        """Called by the statedriver when a state we started is done"""
        pass

    def update(self):
        """Called by the statedriver when it's time to update"""
        pass
    
