#!/usr/bin/env python
"Various support classes are lumped together here"

import constants
import widgets
import traceback

from gui import TransparentGUIState
from widgets import MenuBox
from widgets import Label

def panic(string):
    print "Panic:  %s" % string
    print "Commencing Traceback:"
    traceback.print_stack()
    raise SystemExit


class MenuState(TransparentGUIState):
    """A state which displays a menu and stores the result for whoever
    called it"""
    def __init__(self, driver, background, options, caption = None):
        TransparentGUIState.__init__(self, driver, background)

        container = widgets.BorderedContainer([100,100])
        
        self.menu = MenuBox( (20, 20), 'arrow.png')
        self.menu.yStretch = 0
        self.menu.allowScrolling = 1
        self.menu.resize( (200, 200) )

        self.font = constants.FONT
        font = self.font
        
        for option in options:
            self.menu.add(Label(font, option))

        if caption:
            container.add(widgets.Label(self.font, caption))
        container.add(self.menu)

        self.add(container)

    def callback(self, object):
        self.chosen = object.getChosenIndex()
        self.driver().done()

class Confirm(TransparentGUIState):
    """A state that confirms a given action"""
    def __init__(self, driver, background, caption):
        TransparentGUIState.__init__(self, driver, background)
        self.font = constants.FONT
        self.caption = caption
        self.result = None
        self.container = widgets.BorderedContainer( [200,200] )
        self.menu = MenuBox( (0,0), 'arrow.png')
        self.menu.add(Label(self.font, 'Aye'))
        self.menu.add(Label(self.font, 'Nay'))

        self.container.add(Label(self.font, caption))
        self.container.add(self.menu)

        self.add(self.container)

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['font']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.font = constants.FONT

    def callback(self, object):
        self.result = object.getChosenIndex()
        self.driver().done()

class ShopState(TransparentGUIState):
    """A state that gives the player a list of options, and a price to
    match"""
    def __init__(self, driver, background, caption, options):
        """Options is expected to be a list of tuples of the form
        (option, price).  Background must support a getTown() method
        for getting the town."""
        TransparentGUIState.__init__(self, driver, background)
        self.font = constants.FONT
        self.caption = caption
        self.result = None
        self.container = widgets.BorderedContainer( [100, 100] )
        self.menu = MenuBox( (0,0), 'arrow.png')
        self.town = background.getTown()
        self.options = options
        for option, price in options:
            if price == 0:
                formatted = '$----  %s' % (option)
            else:
                formatted = '$%0.4d  %s' % (price, option)
            item = widgets.Label(self.font, formatted)
            if price > self.town.money:
                item.setColor( (128, 128, 128) )
            self.menu.add(item)

        self.container.add(Label(self.font, caption))
        money = '$%0.4d available' % self.town.money
        self.money = widgets.Label(self.font, money)
        self.container.add(self.money)
        self.container.add(self.menu)

        self.add(self.container)

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['font']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.font = constants.FONT

                           
    def callback(self, object):
        self.result = object.getChosenIndex()
        money = self.options[self.result][1]
        if self.result == len(self.options) - 1:
            # nevermind
            self.result = None
            self.driver().done()
        elif money < self.town.money:
            self.town.money -= money
            self.driver().done()

class NameAsker(TransparentGUIState):
    """A state that asks for a name for something"""
    def __init__(self, driver, background, caption):
        TransparentGUIState.__init__(self, driver, background)
        self.font = constants.FONT
        self.caption = caption
        self.result = None
        self.container = widgets.BorderedContainer( [100, 100] )
        self.input = widgets.Input()
        self.input.resize( (150,30) )

        self.container.add(Label(self.font, caption))
        self.container.add(self.input)

        self.add(self.container)

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['font']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.font = constants.FONT


    def callback(self, object):
        self.result = self.input.text
        self.driver().done()
