#!/usr/bin/env python

import gui
import pygame
import pygame.font

from pygame.constants import *

import constants

class Background(gui.Paintable):
    """Class that simply displays a background image"""
    def __init__(self, filename, visible = True):
        gui.Paintable.__init__(self, visible)
        self.filename = filename
        self._image = pygame.image.load(filename).convert()
        self.aspect = [0, 0, self._image.get_width(),
                       self._image.get_height()]
        
    def paint(self, screen):
        screen.blit(self._image, (0, 0))

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['_image']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self._image = pygame.image.load(self.filename).convert()

class BackgroundColor(gui.Paintable):
    "Class that displays only a background color"
    def __init__(self, color, aspect = (0,0, constants.SCREENSIZE[0],
                                        constants.SCREENSIZE[1]),
                 visible = True):
        gui.Paintable.__init__(self, visible)
        self.color = color
        self._aspect = aspect

    def paint(self, screen):
        screen.fill(self.color)

class Label(gui.Paintable):
    """Displays a string of text"""
    def __init__(self, font, text = '', location= (0,0), visible =
                 True):
        gui.Paintable.__init__(self, visible)
        self._font = font
        self.color = (255, 255, 255)
        self.move(location)
        self.setText(text)

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['_text']
        del odict['_font']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self._font = constants.FONT
        self.setText(dict['text'])

    def setColor(self, color):
        self.color = color
        self.setText(self.text)

    def setText(self, text):
        newSize = self._font.size(text)
        self.resize(newSize)
        self.text = text
        self._text = self._font.render(text, 0, self.color)

    def paint(self, screen):
        screen.blit(self._text, self._aspect)

class BorderedContainer(gui.Container):
    """This arranges all the items placed in it vertically, centered
    horizontally, and draws a background for them"""

    def __init__(self, location, visible = True, enabled = True):
        gui.Container.__init__(self, visible, enabled)
        maxX = constants.SCREENSIZE[0]
        maxY = constants.SCREENSIZE[1]
        w = (maxX / 2 - location[0]) * 2
        h = (maxY / 2 - location[1]) * 2
        self._aspect = [location[0], location[1],w,h]
        self.offset = 3
        self.yPadding = 3
        self.bgcolor = (25, 25, 255)
        self.border = (255, 255, 255)


    def addPaintable(self, paintable):
        gui.Container.addPaintable(self, paintable)

        maxX = constants.SCREENSIZE[0]
        x = maxX / 2 - paintable.size()[0] / 2
        y = self.offset + self.location()[0]
        self.offset += paintable.size()[1] + self.yPadding

        paintable.move( [x,y] )

    def paint(self, screen):

        x = self.getAspect()[0]
        y = self.getAspect()[1]
        w = self.getAspect()[2]
        h = self.getAspect()[3]


        subAspect = [ x + 1, y + 1, w - 2, h - 2 ]
        screen.fill(self.border, self.getAspect())
        screen.fill(self.bgcolor, subAspect)

        screen.set_clip(self.getAspect())

        gui.Container.paint(self, screen)

        screen.set_clip( [ 0, 0, constants.SCREENSIZE[0],
                           constants.SCREENSIZE[1] ] )
                         

class MenuBox(gui.Container):
    """This arranges all the items placed in it and allows the user to
    choose between them.  If enabled, it traps the up/down or
    left/right arrow keys."""

    VERTICAL = 1
    HORIZONTAL = 2
    
    def __init__(self, location, arrowImg, visible = True, enabled =
                 True):
    
        gui.Container.__init__(self, visible, enabled)
        self.move(location)
        self.arrowImg = arrowImg
        self.initialXOffset = 20
        self.initialYOffset = 3
        self.cursorXOffset = 3
        self.cursorYOffset = 3
        self.curXOffset = self.initialXOffset
        self.curYOffset = self.initialYOffset
        self.xSpacing = 3
        self.ySpacing = 3
        self.xStretch = 1
        self.yStretch = 1
        self.align = MenuBox.VERTICAL
        self.bgcolor = (25, 25, 255)
        self.border = (255, 255, 255)
        self.image = pygame.image.load(arrowImg).convert_alpha()
        self.arrowIndex = -1
        self.scrolled = 0
        self.allowScrolling = 0
        self.focuser = None

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['image']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.image = pygame.image.load(self.arrowImg).convert_alpha()

    def addPaintable(self, paintable):
        gui.Container.addPaintable(self, paintable)

        curX = self.getAspect()[0]
        curY = self.getAspect()[1]

        if len(self._paintables) == 1:
            self.arrowIndex = 0

        # place the paintable
        paintable.move( (curX + self.curXOffset, curY +
                         self.curYOffset) )
        if self.align == MenuBox.VERTICAL:
            self.curYOffset += (paintable.getAspect()[3] +
                                self.ySpacing)
            if self.xStretch:
                if (self.curXOffset + paintable.getAspect()[2] >
                    self.getAspect()[2]):
                    xSize = self.curXOffset + paintable.getAspect()[2]
                else:
                    xSize = self.getAspect()[2]
            else: xSize = self.getAspect()[2]

            if self.yStretch:
                if self.curYOffset > self.getAspect()[3]:
                    self.resize( (xSize, self.curYOffset) )
            
        elif self.align == MenuBox.HORIZONTAL:
            self.curXOffset += (paintable.getAspect()[2] +
                                self.xSpacing)

            if self.yStretch:
                if (self.curYOffset + paintable.getAspect()[3] >
                    self.getAspect()[3]):
                    ySize = self.curYOffset + paintable.getAspect()[3]
                else:
                    ySize = self.getAspect()[3]

            if self.xStretch:
                if self.curXOffset > self.getAspect()[2]:
                    self.resize( (self.curXOffset, ySize) )

    def clear(self):
        gui.Container.clear(self)
        self.initialXOffset = 20
        self.initialYOffset = 3
        self.curXOffset = self.initialXOffset
        self.curYOffset = self.initialYOffset
        self.arrowIndex = -1
        self.scrolled = 0

    def callFocus(self, direction):
        """We moved in an unsanctioned direction.  If we have a
        focuser, let's ask it what to do"""
        self.direction = direction
        if self.focuser:
            self.focuser.focusEvent(self)

    def event(self, key, pressed, unicode = None):
        length = len(self._paintables)
        if length:
            if pressed:
                if ((key == K_UP and self.align == MenuBox.VERTICAL) or
                    (key == K_LEFT and self.align ==
                     MenuBox.HORIZONTAL)):
                    self.moveLeft()
                elif ((key == K_DOWN and self.align ==
                       MenuBox.VERTICAL) or
                      (key == K_RIGHT and self.align ==
                       MenuBox.HORIZONTAL)):
                    self.moveRight()

                elif (key == K_DOWN and self.align ==
                      MenuBox.HORIZONTAL):
                    self.callFocus(K_DOWN)
                elif (key == K_UP and self.align ==
                      MenuBox.HORIZONTAL):
                    self.callFocus(K_UP)
                elif (key == K_LEFT and self.align ==
                      MenuBox.VERTICAL):
                    self.callFocus(K_LEFT)
                elif (key == K_RIGHT and self.align ==
                      MenuBox.VERTICAL):
                    self.callFocus(K_RIGHT)
                elif (key == K_RETURN):
                    self.callback()

    def getChosen(self):
        return self._paintables[self.arrowIndex]
        
    def getChosenIndex(self):
        return self.arrowIndex

    def moveLeft(self):
        if (self.scrolled - self.arrowIndex == 0 and
            self.arrowIndex > 0):
            self.scroll(-1)
        
        self.arrowIndex -= 1
        if self.arrowIndex < 0: self.arrowIndex = 0

    def moveRight(self):
        if self.align == MenuBox.VERTICAL:
            paintLength = self._paintables[0].getAspect()[3]
            cursor = (self.initialYOffset + self.arrowIndex *
                      (self.ySpacing + paintLength))
            length = self.size()[1]

        else:
            paintLength = self._paintables[0].getAspect()[2]
            cursor = ((self.initialXOffset + self.arrowIndex *
                       (self.xSpacing + paintLength)) -
                      self.initialXOffset)
            length = self.size()[0]
        if (length - cursor <= paintLength + 32 and
            self.arrowIndex != len(self._paintables) - 1):
            self.scroll(1)
        
        self.arrowIndex += 1
        if self.arrowIndex >= len(self._paintables):
            self.arrowIndex = len(self._paintables) - 1

    def paint(self, screen):
        if self.arrowIndex < 0:
            return

        arrowIndex = self.arrowIndex - self.scrolled
        
        screenWidth = screen.get_clip()[2]
        screenHeight = screen.get_clip()[3]

        x = self.getAspect()[0]
        y = self.getAspect()[1]
        w = self.getAspect()[2]
        h = self.getAspect()[3]

        subAspect = [ x + 1, y + 1, w - 2, h - 2 ]
        
        screen.fill(self.border, self.getAspect())
        screen.fill(self.bgcolor, subAspect)

        if not (self.xStretch and self.yStretch):
            # we're not stretching - clip instead
            screen.set_clip(subAspect)


        for widget in self._paintables:
            widget.paint(screen)

        if self._paintables:

            if self.align == MenuBox.VERTICAL:
                cursorX = x + self.cursorXOffset
                cursorY = y + (self.initialYOffset + arrowIndex *
                               (self.ySpacing +
                                self._paintables[0].getAspect()[3]))
            elif self.align == MenuBox.HORIZONTAL:
                cursorY = y + self.cursorYOffset
                cursorX = x + ((self.initialXOffset + arrowIndex *
                                (self.xSpacing +
                                 self._paintables[0].getAspect()[2])) -
                               self.initialXOffset)

            if self.isEnabled():
                screen.blit(self.image, (cursorX, cursorY))

            # reset clip
            screen.set_clip( [ 0, 0, screenWidth, screenHeight] )

    def scroll(self, direction):
        if self.allowScrolling == 0: return
        self.scrolled += direction

        if self.align == MenuBox.VERTICAL:
            amount = (self._paintables[0].getAspect()[3] +
                      self.ySpacing)
            for item in self._paintables:
                old = item.location()
                new = (old[0], old[1] + amount * -direction)
                item.move(new)
        else:
            amount = (self._paintables[0].getAspect()[2] +
                      self.xSpacing)
            for item in self._paintables:
                old = item.location()
                new = (old[0] + amount * -direction, old[1])
                item.move(new)

class Input(gui.Paintable, gui.Eventable):
    """This acts as an input widget"""
    def __init__(self, visible = True, enabled = True):
        gui.Paintable.__init__(self, visible)
        gui.Eventable.__init__(self, visible)
        self.text = ''
        self.bgcolor = (25, 25, 255)
        self.border = (255,255,255)
        self.color = (255,255,255)
        self.offset =3
        self.yPadding = 3
        self.font = constants.FONT

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['font']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.font = constants.FONT

    def paint(self, screen):
        x = self.getAspect()[0]
        y = self.getAspect()[1]
        w = self.getAspect()[2]
        h = self.getAspect()[3]

        subAspect = [ x + 1, y + 1, w - 2, h - 2 ]
        screen.fill(self.border, self.getAspect())
        screen.fill(self.bgcolor, subAspect)

        screen.set_clip(subAspect)

        if self.text:
            text = self.font.render(self.text, 0, self.color)
            screen.blit(text, subAspect)

    def event(self, key, pressed, unicode = None):
        if pressed:
            if (key >= K_EXCLAIM and
                key <= K_z):
                self.text = self.text + unicode
            if key == K_SPACE:
                self.text = self.text + ' '
            if (key == K_DELETE or
                key == K_BACKSPACE):
                if self.text:
                    self.text = self.text[:len(self.text) - 1]
            if key == K_RETURN:
                self.callback()
